/*
 * Copyright 2020 Esri.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package com.example.app;

import java.nio.charset.StandardCharsets;
import java.util.Calendar;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.layout.StackPane;
import javafx.stage.Stage;

import com.esri.arcgisruntime.ArcGISRuntimeEnvironment;
import com.esri.arcgisruntime.geometry.Geometry;
import com.esri.arcgisruntime.geometry.Polyline;
import com.esri.arcgisruntime.geometry.SpatialReferences;
import com.esri.arcgisruntime.loadable.LoadStatus;
import com.esri.arcgisruntime.location.SimulatedLocationDataSource;
import com.esri.arcgisruntime.location.SimulationParameters;
import com.esri.arcgisruntime.mapping.ArcGISMap;
import com.esri.arcgisruntime.mapping.BasemapStyle;
import com.esri.arcgisruntime.mapping.view.LocationDisplay;
import com.esri.arcgisruntime.mapping.view.MapView;

import org.apache.commons.io.IOUtils;


public class App extends Application {

  private MapView mapView;

  public static void main(String[] args) {
    Application.launch(args);
  }

  @Override
  public void start(Stage stage) {

    try {

      // set the title and size of the stage and show it
      stage.setTitle("Display device location");

      stage.setWidth(800);
      stage.setHeight(700);
      stage.show();

      // create a JavaFX scene with a stack pane as the root node, and add it to the scene
      StackPane stackPane = new StackPane();
      Scene scene = new Scene(stackPane);

      stage.setScene(scene);

      // Note: it is not best practice to store API keys in source code.
      // The API key is referenced here for the convenience of this tutorial.
      String yourApiKey = "YOUR_API_KEY";
      ArcGISRuntimeEnvironment.setApiKey(yourApiKey);

      // create a map view to display the map and add it to the stack pane
      mapView = new MapView();
      stackPane.getChildren().add(mapView);

      ArcGISMap map = new ArcGISMap(BasemapStyle.ARCGIS_TOPOGRAPHIC);

      // set the map on the map view
      mapView.setMap(map);

      // access the json of the location points
      String polylineData = IOUtils.toString(getClass().getResourceAsStream(
              "/polyline_data.json"), StandardCharsets.UTF_8);

      // create a polyline from the location points
      Polyline locations = (Polyline) Geometry.fromJson(polylineData, SpatialReferences.getWgs84());

      // create a simulated location data source
      SimulatedLocationDataSource simulatedLocationDataSource = new SimulatedLocationDataSource();
      // set the location of the simulated location data source with simulation parameters to set a consistent velocity
      simulatedLocationDataSource.setLocations(
              locations, new SimulationParameters(Calendar.getInstance(), 5.0, 0.0, 0.0));

      // configure the map view's location display to follow the simulated location data source
      LocationDisplay locationDisplay = mapView.getLocationDisplay();
      locationDisplay.setLocationDataSource(simulatedLocationDataSource);

      locationDisplay.setAutoPanMode(LocationDisplay.AutoPanMode.RECENTER);
      locationDisplay.setInitialZoomScale(1000);

      map.addDoneLoadingListener(() -> {
        if (map.getLoadStatus() == LoadStatus.LOADED) {

          // start the location display
          locationDisplay.startAsync();

        } else {
          new Alert(Alert.AlertType.ERROR, "Map failed to load: " + map.getLoadError().getCause().getMessage()).show();
        }
      });

    } catch (Exception e) {
      // on any error, display the stack trace.
      e.printStackTrace();
    }

  }

  /**
   * Stops and releases all resources used in application.
   */
  @Override
  public void stop() {

    if (mapView != null) {
      mapView.dispose();
    }
  }

}
